"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = apiThunk;

var _ramda = require("ramda");

var _actions = require("../actions");

var _utils = require("./utils");

var _constants = require("./constants");

var _constants2 = require("../constants/constants");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/* eslint-disable-next-line no-console */
var logWarningOnce = (0, _ramda.once)(console.warn);

function GET(path, fetchConfig) {
  return fetch(path, (0, _ramda.mergeDeepRight)(fetchConfig, {
    method: 'GET',
    headers: (0, _actions.getCSRFHeader)()
  }));
}

function POST(path, fetchConfig) {
  var body = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return fetch(path, (0, _ramda.mergeDeepRight)(fetchConfig, {
    method: 'POST',
    headers: (0, _actions.getCSRFHeader)(),
    body: body ? JSON.stringify(body) : null
  }));
}

var request = {
  GET: GET,
  POST: POST
};

function apiThunk(endpoint, method, store, id, body) {
  return /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(dispatch, getState) {
      var _getState, config, hooks, newHeaders, url, setConnectionStatus, res, retry, _body, newJwt, contentType, message;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              setConnectionStatus = function _setConnectionStatus(connected) {
                if (getState().error.backEndConnected !== connected) {
                  dispatch({
                    type: 'SET_CONNECTION_STATUS',
                    payload: connected
                  });
                }
              };

              _getState = getState(), config = _getState.config, hooks = _getState.hooks;
              newHeaders = null;
              url = "".concat((0, _utils.urlBase)(config)).concat(endpoint);
              dispatch({
                type: store,
                payload: {
                  id: id,
                  status: 'loading'
                }
              });
              _context.prev = 5;
              retry = 0;

            case 7:
              if (!(retry <= _constants.MAX_AUTH_RETRIES)) {
                _context.next = 36;
                break;
              }

              _context.prev = 8;
              _context.next = 11;
              return request[method](url, config.fetch, body);

            case 11:
              res = _context.sent;
              _context.next = 19;
              break;

            case 14:
              _context.prev = 14;
              _context.t0 = _context["catch"](8);
              // fetch rejection - this means the request didn't return,
              // we don't get here from 400/500 errors, only network
              // errors or unresponsive servers.
              console.log('fetch error', res);
              setConnectionStatus(false);
              return _context.abrupt("return");

            case 19:
              if (!(res.status === _constants2.STATUS.UNAUTHORIZED)) {
                _context.next = 32;
                break;
              }

              if (!hooks.request_refresh_jwt) {
                _context.next = 32;
                break;
              }

              _context.next = 23;
              return res.text();

            case 23:
              _body = _context.sent;

              if (!_body.includes(_constants2.JWT_EXPIRED_MESSAGE)) {
                _context.next = 32;
                break;
              }

              _context.next = 27;
              return hooks.request_refresh_jwt(config.fetch.headers.Authorization.substr('Bearer '.length));

            case 27:
              newJwt = _context.sent;

              if (!newJwt) {
                _context.next = 32;
                break;
              }

              newHeaders = {
                Authorization: "Bearer ".concat(newJwt)
              };
              config = (0, _ramda.mergeDeepRight)(config, {
                fetch: {
                  headers: newHeaders
                }
              });
              return _context.abrupt("continue", 33);

            case 32:
              return _context.abrupt("break", 36);

            case 33:
              retry++;
              _context.next = 7;
              break;

            case 36:
              contentType = res.headers.get('content-type');

              if (newHeaders) {
                dispatch((0, _actions.addHttpHeaders)(newHeaders));
              }

              setConnectionStatus(true);

              if (!(contentType && contentType.indexOf('application/json') !== -1)) {
                _context.next = 41;
                break;
              }

              return _context.abrupt("return", res.json().then(function (json) {
                dispatch({
                  type: store,
                  payload: {
                    status: res.status,
                    content: json,
                    id: id
                  }
                });
                return json;
              }));

            case 41:
              logWarningOnce('Response is missing header: content-type: application/json');
              return _context.abrupt("return", dispatch({
                type: store,
                payload: {
                  id: id,
                  status: res.status
                }
              }));

            case 45:
              _context.prev = 45;
              _context.t1 = _context["catch"](5);
              message = 'Error from API call: ' + endpoint;
              (0, _actions.handleAsyncError)(_context.t1, message, dispatch);

            case 49:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[5, 45], [8, 14]]);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
}